﻿Public Enum GeometryType
    Point
    LineString
    Polygon
End Enum

Public Structure MyPoint
    Public X As Double
    Public Y As Double
    Public Z As Double

    Sub New(ByVal nX As Double, ByVal nY As Double, ByVal nZ As Double)
        X = nX
        Y = nY
        Z = nZ
    End Sub

    Sub New(ByVal nX As Double, ByVal nY As Double)
        X = nX
        Y = nY
        Z = Nothing
    End Sub
End Structure

Public MustInherit Class TGeometry

    Public MustOverride Function ToWKT2D() As String
    Public MustOverride Function GetGeometryType() As GeometryType

End Class

Public Class TPoint : Inherits TGeometry

    Public Coord As MyPoint

    Sub New(ByVal nX As Double, ByVal nY As Double, ByVal nZ As Double)
        Coord.X = nX
        Coord.Y = nY
        Coord.Z = nZ
    End Sub

    Sub New(ByVal nX As Double, ByVal nY As Double)
        Coord.X = nX
        Coord.Y = nY
        Coord.Z = Nothing
    End Sub

    Public Overrides Function ToWKT2D() As String
        Return String.Concat("POINT(", Coord.X, " ", Coord.Y, ")")
    End Function

    Public Overrides Function GetGeometryType() As GeometryType
        Return GeometryType.Point
    End Function

End Class

Public Class TLineString : Inherits TGeometry

    Public Coords As Microsoft.VisualBasic.Collection

    Sub New(ByVal PointList As Microsoft.VisualBasic.Collection)
        Coords = PointList
    End Sub

    Public Overrides Function ToWKT2D() As String
        Dim I As Integer
        Dim workStr As String
        Dim workPoint As TPoint
        workStr = "LineString("
        For I = 1 To Coords.Count
            workPoint = Coords.Item(I)
            workStr = String.Concat(workStr, workPoint.Coord.X, " ", workPoint.Coord.Y, ", ")
        Next I
        workPoint = Coords.Item(1)
        workStr = String.Concat(workStr, workPoint.Coord.X, " ", workPoint.Coord.Y, ")")
        Return String.Concat(workStr, ")")
    End Function

    Public Overrides Function GetGeometryType() As GeometryType
        Return GeometryType.LineString
    End Function

End Class

Public Class TPolygon : Inherits TGeometry

    Public Rings As Microsoft.VisualBasic.Collection

    Sub New(ByVal RingList As Microsoft.VisualBasic.Collection)
        Rings = RingList
    End Sub

    Public Overrides Function ToWKT2D() As String
        Dim I, J As Integer
        Dim workStr, workStr1 As String
        Dim workLS As TLineString
        Dim workPoint As TPoint
        workStr = "Polygon("
        For I = 1 To Rings.Count
            workLS = Rings.Item(I)
            workStr1 = "("
            For J = 1 To workLS.Coords.Count
                workPoint = workLS.Coords.Item(J)
                workStr1 = String.Concat(workStr1, workPoint.Coord.X, " ", workPoint.Coord.Y, ", ")
            Next J
            workPoint = workLS.Coords.Item(1)
            workStr1 = String.Concat(workStr1, workPoint.Coord.X, " ", workPoint.Coord.Y, ")")
            workStr = String.Concat(workStr, workStr1)
        Next I
        Return String.Concat(workStr, ")")
    End Function

    Public Overrides Function GetGeometryType() As GeometryType
        Return GeometryType.LineString
    End Function

End Class